==> amsdal_data-0.2.2-cp311-cp311-macosx_10_9_universal2.whl: amsdal_data/utils.py <==
import uuid
from datetime import date
from datetime import datetime
from typing import Any

import amsdal_glue as glue
from amsdal_glue_core.common.interfaces.connection import ConnectionBase
from amsdal_utils.config.data_models.repository_config import RepositoryConfig
from amsdal_utils.models.data_models.enums import CoreTypes
from amsdal_utils.models.data_models.schema import ObjectSchema
from amsdal_utils.utils.classes import import_class
from amsdal_utils.utils.singleton import Singleton

from amsdal_data.connections.constants import METADATA_KEY
from amsdal_data.connections.constants import METADATA_TABLE
from amsdal_data.connections.constants import REFERENCE_TABLE
from amsdal_data.connections.constants import SECONDARY_PARTITION_KEY
from amsdal_data.connections.constants import TRANSACTION_TABLE
from amsdal_data.connections.db_alias_map import CONNECTION_BACKEND_ALIASES


class SchemaManagerBase:
    pass


class SchemaManagerHandler(metaclass=Singleton):
    def __init__(self) -> None:
        self._schema_manager: SchemaManagerBase | None = None

    def set_schema_manager(self, schema_manager: SchemaManagerBase) -> None:
        self._schema_manager = schema_manager

    def get_schema_manager(self) -> SchemaManagerBase:
        if self._schema_manager is None:
            msg = 'Schema manager is not set.'
            raise ValueError(msg)
        return self._schema_manager


def resolve_backend_class(backend: str) -> type[ConnectionBase]:
    if backend in CONNECTION_BACKEND_ALIASES:
        backend = CONNECTION_BACKEND_ALIASES[backend]

    return import_class(backend)


def get_schemas_for_connection_name(connection_name: str, repository_config: RepositoryConfig) -> list[str | None]:
    if connection_name == repository_config.default:
        return [None]

    return [
        _schema_name
        for _schema_name, _connection_name in repository_config.models.items()
        if _connection_name == connection_name
    ]


def object_schema_to_glue_schema(
    object_schema: ObjectSchema,
    *,
    is_lakehouse_only: bool = False,
) -> glue.Schema:
    from amsdal_data.connections.constants import PRIMARY_PARTITION_KEY
    from amsdal_data.connections.historical.data_query_transform import METADATA_FIELD

    _pk_constraint = glue.PrimaryKeyConstraint(
        name=f'pk_{object_schema.title.lower()}_{uuid.uuid4().hex[:8]}',
        fields=[PRIMARY_PARTITION_KEY],
    )
    _schema = glue.Schema(
        name=object_schema.title,
        version=glue.Version.LATEST,
        properties=[
            glue.PropertySchema(
                name=PRIMARY_PARTITION_KEY,
                type=str,
                required=True,
            ),
        ],
        constraints=[
            _pk_constraint,
        ],
        indexes=[],
    )

    if is_lakehouse_only:
        _schema.properties.append(
            glue.PropertySchema(
                name=SECONDARY_PARTITION_KEY,
                type=str,
                required=True,
            ),
        )
        _pk_constraint.fields.append(SECONDARY_PARTITION_KEY)

    for _, property_data in (object_schema.properties or {}).items():
        _property = glue.PropertySchema(
            name=property_data.field_name or '',
            type=object_schema_type_to_glue_type(property_data.type),
            required=property_data.field_name in object_schema.required,
            default=property_data.default,
            description=property_data.title,
        )
        _schema.properties.append(_property)

    for unique_fields in getattr(object_schema, 'unique', []) or []:
        if not _schema.constraints:
            _schema.constraints = []

        _schema.constraints.append(
            glue.UniqueConstraint(
                fields=unique_fields,
                name=f'unq_{object_schema.title}_{"_".join(unique_fields)}',
            ),
        )

    for indexed_field in getattr(object_schema, 'indexed', []) or []:
        if not _schema.indexes:
            _schema.indexes = []

        _schema.indexes.append(
            glue.IndexSchema(
                fields=[indexed_field],
                name=f'idx_{object_schema.title}_{indexed_field}',
            ),
        )

    if is_lakehouse_only or any(_prop.name == METADATA_KEY for _prop in _schema.properties):
        return _schema

    if _schema.name not in (
        METADATA_TABLE,
        REFERENCE_TABLE,
        TRANSACTION_TABLE,
    ):
        _schema.properties.append(
            glue.PropertySchema(
                name=METADATA_FIELD,
                type=dict,
                required=True,
            ),
        )

    return _schema


def object_schema_type_to_glue_type(property_type: str) -> glue.Schema | glue.SchemaReference | type[Any]:
    if property_type == CoreTypes.ANYTHING.value:
        return dict

    if property_type == CoreTypes.NUMBER.value:
        return float

    if property_type == CoreTypes.BOOLEAN.value:
        return bool

    if property_type == CoreTypes.STRING.value:
        return str

    if property_type == CoreTypes.DATE.value:
        return date

    if property_type == CoreTypes.DATETIME.value:
        return datetime

    if property_type == CoreTypes.BINARY.value:
        return bytes

    if property_type == CoreTypes.ARRAY.value:
        return list

    if property_type == CoreTypes.DICTIONARY.value:
        return dict

    if SchemaManagerHandler().get_schema_manager().get_schema_by_name(property_type):  # type: ignore[attr-defined]
        return dict

    if property_type[0] == property_type[0].upper():
        return dict

    return str

==> amsdal_data-0.2.5-cp311-cp311-macosx_10_9_universal2.whl: amsdal_data/utils.py <==
from datetime import date
from datetime import datetime
from typing import Any

import amsdal_glue as glue
from amsdal_glue_core.common.interfaces.connection import ConnectionBase
from amsdal_utils.config.data_models.repository_config import RepositoryConfig
from amsdal_utils.models.data_models.enums import CoreTypes
from amsdal_utils.models.data_models.schema import ObjectSchema
from amsdal_utils.utils.classes import import_class
from amsdal_utils.utils.singleton import Singleton

from amsdal_data.connections.constants import METADATA_KEY
from amsdal_data.connections.constants import METADATA_TABLE
from amsdal_data.connections.constants import REFERENCE_TABLE
from amsdal_data.connections.constants import SECONDARY_PARTITION_KEY
from amsdal_data.connections.constants import TRANSACTION_TABLE
from amsdal_data.connections.db_alias_map import CONNECTION_BACKEND_ALIASES


class SchemaManagerBase:
    pass


class SchemaManagerHandler(metaclass=Singleton):
    def __init__(self) -> None:
        self._schema_manager: SchemaManagerBase | None = None

    def set_schema_manager(self, schema_manager: SchemaManagerBase) -> None:
        self._schema_manager = schema_manager

    def get_schema_manager(self) -> SchemaManagerBase:
        if self._schema_manager is None:
            msg = 'Schema manager is not set.'
            raise ValueError(msg)
        return self._schema_manager


def resolve_backend_class(backend: str) -> type[ConnectionBase]:
    if backend in CONNECTION_BACKEND_ALIASES:
        backend = CONNECTION_BACKEND_ALIASES[backend]

    return import_class(backend)


def get_schemas_for_connection_name(connection_name: str, repository_config: RepositoryConfig) -> list[str | None]:
    if connection_name == repository_config.default:
        return [None]

    return [
        _schema_name
        for _schema_name, _connection_name in repository_config.models.items()
        if _connection_name == connection_name
    ]


def object_schema_to_glue_schema(
    object_schema: ObjectSchema,
    *,
    is_lakehouse_only: bool = False,
) -> glue.Schema:
    from amsdal_data.connections.constants import PRIMARY_PARTITION_KEY
    from amsdal_data.connections.historical.data_query_transform import METADATA_FIELD

    _pk_constraint = glue.PrimaryKeyConstraint(
        name=f'pk_{object_schema.title.lower()}',
        fields=[PRIMARY_PARTITION_KEY],
    )
    _schema = glue.Schema(
        name=object_schema.title,
        version=glue.Version.LATEST,
        properties=[
            glue.PropertySchema(
                name=PRIMARY_PARTITION_KEY,
                type=str,
                required=True,
            ),
        ],
        constraints=[
            _pk_constraint,
        ],
        indexes=[],
    )

    if is_lakehouse_only:
        _schema.properties.append(
            glue.PropertySchema(
                name=SECONDARY_PARTITION_KEY,
                type=str,
                required=True,
            ),
        )
        _pk_constraint.fields.append(SECONDARY_PARTITION_KEY)

    for _, property_data in (object_schema.properties or {}).items():
        _property = glue.PropertySchema(
            name=property_data.field_name or '',
            type=object_schema_type_to_glue_type(property_data.type),
            required=property_data.field_name in object_schema.required,
            default=property_data.default,
            description=property_data.title,
        )
        _schema.properties.append(_property)

    for unique_fields in getattr(object_schema, 'unique', []) or []:
        if not _schema.constraints:
            _schema.constraints = []

        _schema.constraints.append(
            glue.UniqueConstraint(
                fields=unique_fields,
                name=f'unq_{object_schema.title.lower()}_{"_".join(unique_fields)}',
            ),
        )

    for indexed_field in getattr(object_schema, 'indexed', []) or []:
        if not _schema.indexes:
            _schema.indexes = []

        _schema.indexes.append(
            glue.IndexSchema(
                fields=[indexed_field],
                name=f'idx_{object_schema.title.lower()}_{indexed_field}',
            ),
        )

    if is_lakehouse_only or any(_prop.name == METADATA_KEY for _prop in _schema.properties):
        return _schema

    if _schema.name not in (
        METADATA_TABLE,
        REFERENCE_TABLE,
        TRANSACTION_TABLE,
    ):
        _schema.properties.append(
            glue.PropertySchema(
                name=METADATA_FIELD,
                type=dict,
                required=True,
            ),
        )

    return _schema


def object_schema_type_to_glue_type(property_type: str) -> glue.Schema | glue.SchemaReference | type[Any]:
    if property_type == CoreTypes.ANYTHING.value:
        return dict

    if property_type == CoreTypes.NUMBER.value:
        return float

    if property_type == CoreTypes.BOOLEAN.value:
        return bool

    if property_type == CoreTypes.STRING.value:
        return str

    if property_type == CoreTypes.DATE.value:
        return date

    if property_type == CoreTypes.DATETIME.value:
        return datetime

    if property_type == CoreTypes.BINARY.value:
        return bytes

    if property_type == CoreTypes.ARRAY.value:
        return list

    if property_type == CoreTypes.DICTIONARY.value:
        return dict

    if property_type[0] == property_type[0].upper():
        return dict

    return str
